"""Preprocess imagery in RAM for use in convolutional neural nets."""
import numpy as np
from osgeo import gdal
from scipy.io import loadmat


class Image_reader:
    """Read the imagery into RAM."""

    def __init__(self, train_dir, reference_dir, is_training=False,
                 nodata_img=[], nodata_gt=0):
        """Initialise the class with required data."""
        self.is_training = is_training
        self.train_dir = train_dir
        self.nodata_img = nodata_img
        if self.is_training:
            self.reference_dir = reference_dir
            self.nodata_gt = nodata_gt
        pass


class Image_tiler:
    """Tile imagery in RAM for use in convolutional neural nets."""

    def __init__(self, in_arr, out_shape=(256, 256),
                 out_overlap=128, offset=(0, 0)):
        """
        Initialize the class with required data.

        in_arr:         the numpy array to tile
        out_shape:      tuple of (height, width) of resulting tiles
        out_overlap:    int, number of pixels to overlap by
        offset:         tuple, offset from top left corner in pixels
        """
        self.in_arr = in_arr
        self.in_shape = in_arr.shape
        self.out_shape = out_shape
        self.out_overlap = out_overlap
        self.offset = offset

    def crop_image(self):
        """Crops the input image in order to be tileable."""
        height = self.out_shape[0] + self.offset[0]
        while True:
            height += (self.out_shape[0] - self.out_overlap)
            if self.in_shape[0] < height:
                height -= (self.out_shape[0] - self.out_overlap)
                break

        width = self.out_shape[1] + self.offset[1]
        while True:
            width += (self.out_shape[1] - self.out_overlap)
            if self.in_shape[1] < width:
                width -= (self.out_shape[1]-self.out_overlap)
                break

        self.crop_arr = self.in_arr[self.offset[0]:height,
                                    self.offset[1]:width, :]
        return self.crop_arr

    def tile_image(self):
        """Tiles the input image in order to use it in CNNs."""
        self.tiles_num_ver = int((self.in_shape[0] - self.out_shape[0])
                                 / (self.out_shape[0] - self.out_overlap)) + 1
        self.tiles_num_hor = int((self.in_shape[1] - self.out_shape[1])
                                 / (self.out_shape[1] - self.out_overlap)) + 1

        tiles_num = self.tiles_num_ver * self.tiles_num_hor
        self.tiles_arr = np.empty((tiles_num, self.out_shape[0],
                                  self.out_shape[1], self.in_shape[2]),
                                  self.in_arr.dtype)
        idx = 0

        for row in range(self.tiles_num_ver):
            for col in range(self.tiles_num_hor):
                row_start = row * (self.out_shape[0] - self.out_overlap)
                col_start = col * (self.out_shape[1] - self.out_overlap)
                self.tiles_arr[idx, :, :, :] = self.in_arr[
                    row_start:row_start+self.out_shape[0],
                    col_start:col_start+self.out_shape[1], :]
                idx += 1
        return self.tiles_arr


def run_tiling(in_arr, out_shape=(256, 256), out_overlap=128, offset=(0, 0)):
    """Tile the image."""
    dataset = Image_tiler(in_arr, out_shape, out_overlap, offset)
    dataset.crop_image()
    dataset.tile_image()
    tiles_arr = dataset.tiles_arr
    del dataset
    return tiles_arr


def return_dimensions(in_arr, out_shape=(256, 256), out_overlap=128,
                      offset=(0, 0)):
    """Return the tiled dimensions."""
    dataset = Image_tiler(in_arr, out_shape, out_overlap, offset)
    dataset.crop_image()
    dataset.tile_image()

    dims = {
        'tiles_num': (dataset.tiles_num_ver, dataset.tiles_num_hor),
        'cropped_shape': dataset.crop_arr.shape
    }
    del dataset
    return dims


def read_gdal(trainingdata_path, referencedata_path):
    """Read data using gdal and transform into numpy array."""
    raster_orig = gdal.Open(trainingdata_path)
    raster_orig_arr = np.moveaxis(raster_orig.ReadAsArray(), 0, -1)
    raster_orig = None

    raster_gt = gdal.Open(referencedata_path)
    raster_gt_arr = np.expand_dims(raster_gt.ReadAsArray(), axis=-1)
    raster_gt = None

    arr_dict = {'imagery': raster_orig_arr,
                'reference': raster_gt_arr}
    return arr_dict


def read_gdal_with_geoinfo(trainingdata_path, offset=(0, 0)):
    """Read data using gdal and transform into numpy array."""
    def retrieve_geoinfo(img, offset):
        old_geot = img.GetGeoTransform()
        new_geot = (old_geot[0] + old_geot[1] * offset[0],
                    old_geot[1],
                    old_geot[2],
                    old_geot[3] + old_geot[5] * offset[1],
                    old_geot[4],
                    old_geot[5])
        geoinfo = {
            'projection': img.GetProjection(),
            'geotransform': new_geot
            }
        return geoinfo

    raster_orig = gdal.Open(trainingdata_path)
    raster_orig_arr = np.moveaxis(raster_orig.ReadAsArray(), 0, -1)
    raster_orig_geoinfo = retrieve_geoinfo(raster_orig, offset)
    raster_orig = None

    arr_dict = {'imagery': raster_orig_arr, 'geoinfo': raster_orig_geoinfo}
    return arr_dict


def read_pavia_centre(train_path, ref_path=None, out_shape=(1096, 1096, 102)):

    raster_orig = loadmat(train_path)
    raster_orig_arr = raster_orig['pavia']
    imagery = np.zeros(out_shape, dtype=np.uint16)
    imagery[:, :223, :] = raster_orig_arr[:out_shape[0], :223, :out_shape[2]]
    imagery[:, 605-(1096-out_shape[1]):, :] = raster_orig_arr[
        :out_shape[0], 224:, :out_shape[2]]
    arr_dict = {'imagery': imagery}

    if ref_path is not None:
        raster_gt = loadmat(ref_path)
        raster_gt_arr = raster_gt['pavia_gt'][:, :, None]
        reference = np.zeros([out_shape[0], out_shape[1], 1], dtype=np.uint8)
        reference[:, :223, :] = raster_gt_arr[:out_shape[0], :223, :]
        reference[:, 605-(1096-out_shape[1]):, :] = raster_gt_arr[
            :out_shape[0], 224:, :]
        arr_dict['reference'] = reference

    arr_dict['geoinfo'] = {}
    return arr_dict


def run_tiling_dims(in_arr, out_shape=(256, 256), out_overlap=128,
                    offset=(0, 0)):
    """Tile the image."""
    dataset = Image_tiler(in_arr, out_shape, out_overlap, offset)
    dataset.crop_image()
    dataset.tile_image()

    tiles_arr = dataset.tiles_arr
    dims = {
        'tiles_num': (dataset.tiles_num_ver, dataset.tiles_num_hor),
        'cropped_shape': dataset.crop_arr.shape
    }

    del dataset
    out_dict = {'imagery': tiles_arr, 'dimensions': dims}
    return out_dict


def tile_training(in_dict, shape, overlap, offset=(0, 0)):
    """Tile the imagery for training."""
    tiled_imagery = run_tiling_dims(in_dict['imagery'], out_shape=shape,
                                    out_overlap=overlap, offset=offset)
    tiled_reference = run_tiling_dims(in_dict['reference'], out_shape=shape,
                                      out_overlap=overlap, offset=offset)
    out_dict = {
        'imagery': tiled_imagery['imagery'],
        'reference': tiled_reference['imagery']}
    return out_dict


def filter_useful_tiles(arr_dict, nodata_vals=[], min_usable_area=1.,
                        is_training=False, nodata_gt=0):
    """Filter only tiles with reference information."""
    idxs_keep = []

    if is_training:
        for idx, tile in enumerate(zip(arr_dict['imagery'],
                                       arr_dict['reference'])):
            ref_unique = np.unique(tile[1]).tolist()
            if nodata_gt in ref_unique:
                ref_unique.remove(nodata_gt)
                if len(ref_unique) > 0:
                    idxs_keep.append(idx)
            else:
                idxs_keep.append(idx)

        arr_dict = {
            'imagery': arr_dict['imagery'][idxs_keep],
            'reference': arr_dict['reference'][idxs_keep]
            }

    else:
        for idx, tile in enumerate(arr_dict['imagery']):
            ref_unique = np.unique(tile).tolist()
            if nodata_vals[0] in ref_unique:
                ref_unique.remove(nodata_vals[0])
                if len(ref_unique) > 0:
                    idxs_keep.append(idx)
            else:
                idxs_keep.append(idx)

        arr_dict = {'imagery': arr_dict['imagery'][idxs_keep]}

    return arr_dict


def normalize_tiles(in_dict, nodata_vals=[], is_training=False):
    """Normalize values between 0 and 1."""
    def normalize_input(arr):
        # normalize all values in array
        arr_norm = (arr - arr.min()) / (arr.max() - arr.min())
        return arr_norm
    arr_dict = {}
    arr_dict['imagery'] = in_dict['imagery'].transpose([0, 3, 1, 2])
    for nodata_val in nodata_vals:
        arr_dict['imagery'][arr_dict['imagery'] == nodata_val] = 0
    arr_dict['imagery'] = normalize_input(arr_dict['imagery']
                                          .astype(np.float32))
    if is_training:
        ref_newshape = in_dict['reference'].shape[:-1]
        arr_dict['reference'] = in_dict['reference'].reshape(ref_newshape)
        arr_dict['reference'] = arr_dict['reference'].astype(np.int64)
        unique, counts = np.unique(arr_dict['reference'], return_counts=True)
        return arr_dict, unique, counts
    else:
        return arr_dict


def normalize_tiles_1d(in_dict, nodata_vals=[], is_training=False):
    """Normalize values between 0 and 1."""
    arr_dict = {}
    if is_training:
        norm_dict, _, _ = normalize_tiles(in_dict, nodata_vals, is_training)
        arr_dict['imagery'] = norm_dict['imagery'][:, None, :, 0, 0]
        arr_dict['reference'] = norm_dict['reference'][:, 0, 0]
        arr_dict['reference'] = arr_dict['reference'] - 1
        unique, counts = np.unique(arr_dict['reference'], return_counts=True)
        return arr_dict, unique, counts
    else:
        norm_dict = normalize_tiles(in_dict, nodata_vals, is_training)
        arr_dict['imagery'] = norm_dict['imagery'][:, None, :, 0, 0]
        return arr_dict


def normalize_tiles_3d(in_dict, nodata_vals=[], is_training=False):
    """Normalize values between 0 and 1."""
    arr_dict = {}
    if is_training:
        norm_dict, _, _ = normalize_tiles(in_dict, nodata_vals, is_training)
        arr_dict['imagery'] = norm_dict['imagery'][:, None, :, :, :]
        arr_dict['reference'] = norm_dict['reference']
        unique, counts = np.unique(arr_dict['reference'], return_counts=True)
        return arr_dict, unique, counts
    else:
        norm_dict = normalize_tiles(in_dict, nodata_vals, is_training)
        arr_dict['imagery'] = norm_dict['imagery'][:, None, :, :, :]
        return arr_dict


"""
if __name__ == '__main__':
    dummy_filename = 'C:\\Users\\dd\\Pictures\\DSC_0084.jpg'
    dummy_arr = imageio.imread(dummy_filename)  # .astype(np.float32)

    print(dummy_arr.dtype)
    print(dummy_arr.shape)

    dummy_dataset = Image_tiler(dummy_arr, (256, 256), 128, (0, 0))
    dummy_crop = dummy_dataset.crop_image()
    print(dummy_crop.shape)
    dummy_tiles = dummy_dataset.tile_image()
    print(dummy_tiles.shape)
"""



#
# ------------------------------------------------------------------------------
# Copyright (c) 2013-2014, Christian Therien
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------
#


"""
SAM_classifier, SID_classifier functions
Spectral Angle Mapper class
Spectral Information Divergence class
"""


import numpy as np


def _normalize(M):
    """
    Normalizes M to be in range [0, 1].

    Parameters:
      M: `numpy array`
          1D, 2D or 3D data.

    Returns: `numpy array`
          Normalized data.
    """

    minVal = np.min(M)
    maxVal = np.max(M)

    Mn = M - minVal

    if maxVal == minVal:
        return np.zeros(M.shape)
    else:
        return Mn / (maxVal-minVal)


def _single_value_min(data, threshold):
    """
    Use a threshold to extract the minimum value along
    the data y axis.
    """
    min_vec = np.min(data, axis=1)
    amin = np.min(data)
    amax = np.max(data)
    limit = amin + (amax - amin) * threshold
    min_mask = min_vec < limit
    argmin = np.argmin(data, axis=1)
    return (argmin + 1) * min_mask


# For the SAM_classifier function, if you have these errors messages:
"""
C:/somepath/pysptools/classifiers/hs_classification.py:98:
  RuntimeWarning: invalid value encountered in arccos
  angles = np.arccos(np.round(sum_T_R / mul_T_R, _round_threshold))
C:/somepath/pysptools/classifiers/hs_classification.py:19:
  RuntimeWarning: invalid value encountered in less
  min_mask = min_vec < limit
"""
# ajust the _round_threshold parameter accordingly,
# a smaller value avoid these errors.
_round_threshold = 7


def SAM_classifier(M, E, threshold=0.1):
    """
    Classify a HSI cube M using the spectral angle mapper
    and a spectral library E
    This function is part of the SAM class.

    Parameters
        M : numpy array
          a HSI array (m x p)

        E : numpy array
          a spectral library (N x p)

        threshold : float
          threshold for classification (0-1)

    Returns : numpy array
          a class map (m)
    """
    def norm_array(m):
        res = np.zeros(m.shape[0])
        for i in range(m.shape[0]):
            res[i] = np.dot(m[i], m[i])
        return np.sqrt(res)

    M = _normalize(M)
    E = _normalize(E)

    TNA = norm_array(M)
    RNA = norm_array(E)
    sum_T_R = np.dot(E, M.T).T
    mul_T_R = np.ndarray((TNA.shape[0], RNA.shape[0]), dtype=float)
    for i in range(TNA.shape[0]):
        mul_T_R[i] = np.multiply(TNA[i], RNA)
    # read above for _round_threshold
    angles = np.arccos(np.round(sum_T_R / mul_T_R, _round_threshold))
    if isinstance(threshold, float):
        cmap = _single_value_min(angles, threshold)
    else:
        return np.argmin(angles, axis=1), angles
    return cmap


def SID_classifier(M, E, threshold=0.1):
    """
    Classify a HSI cube M using the spectral information divergence
    and a spectral library E.

    Parameters
        M : numpy array
          a HSI array (m x p)

        E : numpy array
          a spectral library (N x p)

        threshold : float
          threshold for classification (0-1)

    Returns : numpy array
          a class map (m)
    """
    def prob_vector_array(m):
        pv_array = np.ndarray(shape=m.shape, dtype=float)
        sum_m = np.sum(m, axis=1)
        pv_array[:] = (m.T / sum_m).T
        return pv_array + np.spacing(1)

    mn = M.shape[0]
    N = E.shape[0]
    p = prob_vector_array(M)
    q = prob_vector_array(E)
    sid = np.ndarray((mn, N), dtype=float)
    for i in range(mn):
        pq = q[0:, :] * np.log(q[0:, :] / p[i, :])
        pp = p[i, :] * np.log(p[i, :] / q[0:, :])
        sid[i, :] = np.sum(pp[0:, :] + pq[0:, :], axis=1)
    if isinstance(threshold, float):
        cmap = _single_value_min(sid, threshold)
    else:
        return np.argmin(sid, axis=1), sid
    return cmap



"""Set of functions for visualisations."""
import numpy as np
import torch
import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap
from time import gmtime, strftime
from sklearn.metrics import confusion_matrix


def _create_colorlist_classnames(arr=None, ds_name='pavia_centre'):
    """Return correct colormap and class names for plotting."""
    if ds_name == 'pavia_centre':
        color_list = ('white', 'blue', 'green', 'olive', 'red',
                      'yellow', 'grey', 'cyan', 'orange', 'black')
        class_names = ('No Data', 'Water', 'Trees', 'Meadows',
                       'Self-Blocking Bricks', 'Bare Soil', 'Asphalt',
                       'Bitumen', 'Tiles', 'Shadows')

    elif ds_name == 'lucni_hora':
        color_list = ('white', 'red', 'green', 'yellow', 'orange', 'purple',
                      'blue', 'cyan', 'black', 'grey')
        class_names = ('No Data', 'Metlička Křivolaká',
                       'metlička, tomka a ostřice', 'brusnice borůvková',
                       'metlice trsnatá', 'borovice kleč', 'smilka tuhá',
                       'kamenná moře bez vegetace', 'vřes obecný',
                       'kameny, půda, mechy a vegetace')

    elif ds_name == 'bila_louka':
        color_list = ('white', 'red', 'green', 'yellow',
                      'orange', 'purple', 'blue', 'grey')
        class_names = ('No Data', 'afs', 'cv', 'cxbig',
                       'desch', 'mol', 'nard', 'smrk')

    else:
        print('Incorrect dataset name for creating a plot. Cannot create a \
              colormap and a list of class names.')
        print('Valid dataset names are "lucni_hora", "bila_louka" or \
              "pavia_centre".')

    if type(arr) is np.ndarray:
        arr_min, arr_max = np.min(arr), np.max(arr)
        out_cmap = color_list[arr_min:arr_max+1]
        out_class_names = class_names[arr_min:arr_max+1]
        return out_cmap, out_class_names

    else:
        return color_list, class_names


def _image_show(raster, title='Natural color composite'):
    """Show a figure based on a hyperspectral raster."""
    plt.imshow(raster/3000)
    plt.title(title)
    plt.axis('off')


def _class_show(raster, title, ds_name='pavia_centre'):
    """Show a figure based on a classification."""
    colorlist, classnames = _create_colorlist_classnames(raster,
                                                         ds_name=ds_name)
    for label, color in zip(classnames, colorlist):
        plt.plot(0, 0, 's', label=label, alpha=1,
                 color=color, markeredgecolor='black')

    plt.imshow(raster, cmap=ListedColormap(colorlist), interpolation='nearest')
    # plt.colorbar(ticks=(np.linspace(0.5, 8.5, 10)))
    plt.title(title)
    plt.axis('off')
    # plt.legend()


def show_img_ref(hs_img, gt_img, ds_name='pavia_centre'):
    """Show the hyperspectral image and a training reference."""
    plt.figure(figsize=[16, 8])
    plt.subplot(1, 2, 1)
    _image_show(hs_img)
    plt.subplot(1, 2, 2)
    _class_show(gt_img, 'Reference data', ds_name=ds_name)
    plt.legend(ncol=3, bbox_to_anchor=(0.5, -0.15), loc='lower center')


def show_spectral_curve(tile_dict, tile_num, ds_name='pavia_centre',
                        title='Spectral curve for pixel #'):
    """Show a figure of the spectral curve."""
    # Choose a range of collected wavelengths
    if ds_name == 'krkonose':
        wl_min, wl_max = 404, 997
        plt.xlabel('Wavelength [nm]')
    else:
        wl_min, wl_max = 1, tile_dict["imagery"].shape[-1] + 1
        plt.xlabel('Band number')
    # Create a vector of wavelength values
    x = np.linspace(wl_min, wl_max, tile_dict["imagery"].shape[-1])

    # Read the spectral curve
    if len(tile_dict["imagery"].shape) == 4:
        y = tile_dict["imagery"][tile_num, 0, 0, :]
        lbl = tile_dict["reference"][tile_num, 0, 0, :][0] + 1
    elif len(tile_dict["imagery"].shape) == 3:
        y = tile_dict["imagery"][tile_num, 0, :]
        lbl = tile_dict["reference"][tile_num] + 1
    elif len(tile_dict["imagery"].shape) == 2:
        y = tile_dict["imagery"][tile_num, :]
        lbl = tile_dict["reference"][tile_num]
    else:
        print('The input data is in an incompatible shape.')

    _, classnames = _create_colorlist_classnames(ds_name=ds_name)

    plt.plot(x, y, label=f'{classnames[lbl]}')
    plt.title(f'{title} {tile_num}')
    plt.legend(bbox_to_anchor=(0.5, 0.89), loc='lower center')


def show_augment_spectral(tile_dict, tile_num, aug_funct,
                          ds_name='pavia_centre'):
    """Show a figure of the original spectal curve and the augmented curve."""
    plt.figure(figsize=[8, 4])
    plt.subplot(1, 2, 1)
    show_spectral_curve(tile_dict, tile_num, ds_name=ds_name,
                        title='Original spectral curve for pixel #')
    plt.subplot(1, 2, 2)
    tensor_obs = torch.from_numpy(tile_dict["imagery"])
    tensor_gt = torch.from_numpy(tile_dict["reference"])
    aug_obs, aug_gt = aug_funct(tensor_obs, tensor_gt)
    aug_dict = {'imagery': aug_obs, 'reference': aug_gt}
    show_spectral_curve(aug_dict, tile_num, ds_name=ds_name,
                        title='Augmented spectral curve for pixel #')


def show_augment_spatial(tile_dict, tile_num, aug_funct,
                         ds_name='pavia_centre'):
    """Show a figure of the original and the augmented RGB composite."""
    img_rgb = tile_dict['imagery'][tile_num, [25, 15, 5], :, :]
    img_rgb_transposed = img_rgb.transpose((1, 2, 0))
    tile_gt = tile_dict['reference'][tile_num, :, :]
    plt.figure(figsize=[20, 5])

    plt.subplot(1, 4, 1)
    _image_show(img_rgb_transposed*20000, title='Original RGB composite')

    plt.subplot(1, 4, 2)
    img_hs = tile_dict['imagery'][tile_num, :, :, :]
    img_aug, gt_aug = aug_funct(torch.from_numpy(img_hs),
                                torch.from_numpy(tile_gt[None, :, :]))
    print(img_aug.shape)
    img_augmented_np = np.array(img_aug)
    img_aug_trans = img_augmented_np[0, [25, 15, 5], :, :].transpose(1, 2, 0)

    _image_show(np.array(img_aug_trans)*20000, title='Augmented RGB composite')

    plt.subplot(1, 4, 3)
    _class_show(tile_gt, 'Original reference data', ds_name=ds_name)
    plt.legend(ncol=3, bbox_to_anchor=(0.5, -0.15), loc='lower center')

    plt.subplot(1, 4, 4)
    _class_show(np.array(gt_aug[0, :, :]), 'Augmented reference data',
                ds_name=ds_name)


def show_augment_spectro_spatial(tile_dict, tile_num, aug_funct,
                                 ds_name='pavia_centre'):
    """Show a figure of the original and the augmented RGB composite."""
    img_rgb = tile_dict['imagery'][tile_num, 0, [25, 15, 5], :, :]
    img_rgb_transposed = img_rgb.transpose((1, 2, 0))
    tile_gt = tile_dict['reference'][tile_num, :, :]
    plt.figure(figsize=[20, 5])

    plt.subplot(1, 4, 1)
    _image_show(img_rgb_transposed*20000, title='Original RGB composite')

    plt.subplot(1, 4, 2)
    img_hs = tile_dict['imagery'][tile_num, 0, :, :, :]
    img_aug, gt_aug = aug_funct(torch.from_numpy(img_hs),
                                torch.from_numpy(tile_gt[None, :, :]))
    print(img_aug.shape)
    img_aug_np = np.array(img_aug)
    img_aug_trans = img_aug_np[0, 0, [25, 15, 5], :, :].transpose(1, 2, 0)

    _image_show(np.array(img_aug_trans)*20000, title='Augmented RGB composite')

    plt.subplot(1, 4, 3)
    _class_show(tile_gt, 'Original reference data', ds_name=ds_name)
    plt.legend(ncol=3)

    plt.subplot(1, 4, 4)
    _class_show(np.array(gt_aug[0, :, :]), 'Augmented reference data',
                ds_name=ds_name)


def show_classified(hs_img, gt_img, class_img, ds_name='pavia_centre'):
    """Compare the classification result to the reference data."""
    plt.figure(figsize=[15, 5])
    plt.subplot(1, 3, 1)
    _image_show(hs_img)

    plt.subplot(1, 3, 2)
    _class_show(gt_img, 'Reference data', ds_name=ds_name)
    plt.legend(ncol=3)

    plt.subplot(1, 3, 3)
    _class_show(class_img, 'Classified data', ds_name=ds_name)


def show_confusion_matrix(gt, predict, ds_name='pavia_centre'):
    conf_matrix = confusion_matrix(gt, predict)

    fig, ax = plt.subplots(figsize=(7.5, 7.5))
    ax.matshow(conf_matrix, cmap=plt.cm.hot_r, alpha=0.3)
    # Add numbers to each position
    for i in range(conf_matrix.shape[0]):
        for j in range(conf_matrix.shape[1]):
            ax.text(x=j, y=i, s=conf_matrix[i, j], va='center', ha='center')

    # Make it look good
    ax.set_xlabel('Predicted labels')
    ax.xaxis.set_label_position('top')
    ax.set_ylabel('Reference labels')
    ax.set_title('Confusion matrix')

    _, classnames = _create_colorlist_classnames(ds_name=ds_name)
    classnames = classnames[-len(np.unique(predict)):]

    ax.set_xticks(np.arange(len(classnames)), labels=classnames)
    ax.set_yticks(np.arange(len(classnames)), labels=classnames)
    ax.tick_params(axis='x', labelrotation=90)

    plt.show()


def sec_to_hms(sec):
    """Convert seconds to hours, minutes, seconds."""
    ty_res = gmtime(sec)
    res = strftime("%Hh, %Mm, %Ss", ty_res)
    return str(res)
